#ifndef SC_MODULE_H
#define SC_MODULE_H

#include <scoot_tags.h>

#include <sc_sensitive.h>
#include <sc_signal.h>
#include <sc_event.h>

namespace sc_core
{

class sc_module
{
    public:

    // Constructors

    sc_module()
    {
        scoot_module_decl(this);
    }
    sc_module(const char* );
;

private:

    // Disabled

    sc_module( const sc_module& );
    sc_module& operator=(const sc_module&);
};


// -----------------------------------------------------------------------------
// WAIT FUNCTIONS:
// -----------------------------------------------------------------------------

void wait();

void wait(const sc_event& e);

inline void wait(int n)
{
    for( int i = 0; i < n; i++)
    {
        wait();
    }
}


// -----------------------------------------------------------------------------
// SOME MACROS TO SIMPLIFY SYNTAX:
// -----------------------------------------------------------------------------

#define SC_MODULE(user_module_name)                                           \
    struct user_module_name : ::sc_core::sc_module

#define SC_CTOR(user_module_name)                                             \
    typedef user_module_name SC_CURRENT_USER_MODULE;                          \
    private:                                                                  \
    user_module_name(const user_module_name&);                                \
    user_module_name& operator = (const user_module_name&);                   \
    public:                                                                   \
    user_module_name(const char*)

// the SC_HAS_PROCESS macro call must be followed by a ;
#define SC_HAS_PROCESS(user_module_name)                                      \
    typedef user_module_name SC_CURRENT_USER_MODULE

//-----------------------------------------------------------------------------
// PROCESS DECLARATION
//-----------------------------------------------------------------------------

template <class T>
void sc_thread(T* pobject, void (T::* pmethod)());

template <class T>
void sc_cthread(T* pobject, void (T::* pmethod)());

template <class T>
void sc_method(T* pobject, void (T::* pmethod)());


#define SC_CTHREAD(handle, edge)                                              \
{                                                                             \
    sc_cthread<SC_CURRENT_USER_MODULE>(this, &SC_CURRENT_USER_MODULE::handle);\
    sensitive << (edge);                                                      \
}                                                                             \

#define SC_THREAD(handle)                                                     \
{                                                                             \
    sc_thread<SC_CURRENT_USER_MODULE>(this, &SC_CURRENT_USER_MODULE::handle);\
}

#define SC_METHOD(handle)                                                     \
{                                                                             \
    sc_method<SC_CURRENT_USER_MODULE>(this, &SC_CURRENT_USER_MODULE::handle);\
}

//-----------------------------------------------------------------------------
// START SIMULATION
//-----------------------------------------------------------------------------

inline void sc_start(int time)
{
    // default time resolution + 1 for initialization
    scoot_start(2*time+1);
}

inline void sc_start()
{
    scoot_start(-1);
}

} // namespace sc_core

#endif
